@@ -6,9 +6,7 @@
 # The contents of this file are pickled, so don't put values in the namespace
 # that aren't pickleable (module imports are okay, they're removed automatically).
 
-import os
-import sys
-import time
+import sys, os, time
 sys.path.append(os.path.abspath('tools/extensions'))
 sys.path.append(os.path.abspath('includes'))
 
@@ -40,23 +38,24 @@ try:
     import _tkinter
 except ImportError:
     _tkinter = None
+# Treat warnings as errors, done here to prevent warnings in Sphinx code from
+# causing spurious test failures.
+import warnings
+warnings.simplefilter('error')
+del warnings
 '''
 
 manpages_url = 'https://manpages.debian.org/{path}'
 
 # General substitutions.
 project = 'Python'
-copyright = f"2001-{time.strftime('%Y')}, Python Software Foundation"
+copyright = '2001-%s, Python Software Foundation' % time.strftime('%Y')
 
 # We look for the Include/patchlevel.h file in the current Python source tree
 # and replace the values accordingly.
 import patchlevel
 version, release = patchlevel.get_version_info()
 
-rst_epilog = f"""
-.. |python_version_literal| replace:: ``Python {version}``
-"""
-
 # There are two options for replacing |today|: either, you set today to some
 # non-false value, then it is used:
 today = ''
@@ -69,10 +68,6 @@ highlight_language = 'python3'
 # Minimum version of sphinx required
 needs_sphinx = '4.2'
 
-# Create table of contents entries for domain objects (e.g. functions, classes,
-# attributes, etc.). Default is True.
-toc_object_entries = False
-
 # Ignore any .rst files in the includes/ directory;
 # they're embedded in pages but not rendered individually.
 # Ignore any .rst files in the venv/ directory.
@@ -84,29 +79,22 @@ if venvdir is not None:
 nitpick_ignore = [
     # Standard C functions
     ('c:func', 'calloc'),
-    ('c:func', 'ctime'),
     ('c:func', 'dlopen'),
     ('c:func', 'exec'),
     ('c:func', 'fcntl'),
-    ('c:func', 'flock'),
     ('c:func', 'fork'),
     ('c:func', 'free'),
-    ('c:func', 'gettimeofday'),
     ('c:func', 'gmtime'),
-    ('c:func', 'grantpt'),
-    ('c:func', 'ioctl'),
-    ('c:func', 'localeconv'),
     ('c:func', 'localtime'),
     ('c:func', 'main'),
     ('c:func', 'malloc'),
-    ('c:func', 'mktime'),
     ('c:func', 'printf'),
     ('c:func', 'realloc'),
     ('c:func', 'snprintf'),
     ('c:func', 'sprintf'),
     ('c:func', 'stat'),
-    ('c:func', 'strftime'),
     ('c:func', 'system'),
+    ('c:func', 'time'),
     ('c:func', 'vsnprintf'),
     # Standard C types
     ('c:type', 'FILE'),
@@ -123,6 +111,8 @@ nitpick_ignore = [
     ('c:type', 'uintptr_t'),
     ('c:type', 'va_list'),
     ('c:type', 'wchar_t'),
+    ('c:type', '__int64'),
+    ('c:type', 'unsigned __int64'),
     # Standard C structures
     ('c:struct', 'in6_addr'),
     ('c:struct', 'in_addr'),
@@ -167,13 +157,6 @@ nitpick_ignore = [
     ('envvar', 'USER'),
     ('envvar', 'USERNAME'),
     ('envvar', 'USERPROFILE'),
-    # Deprecated function that was never documented:
-    ('py:func', 'getargspec'),
-    ('py:func', 'inspect.getargspec'),
-    # Undocumented modules that users shouldn't have to worry about
-    # (implementation details of `os.path`):
-    ('py:mod', 'ntpath'),
-    ('py:mod', 'posixpath'),
 ]
 
 # Temporary undocumented names.
@@ -249,13 +232,6 @@ nitpick_ignore += [
     # be resolved, as the method is currently undocumented. For context, see
     # https://github.com/python/cpython/pull/103289.
     ('py:meth', '_SubParsersAction.add_parser'),
-    # Attributes/methods/etc. that definitely should be documented better,
-    # but are deferred for now:
-    ('py:attr', '__annotations__'),
-    ('py:meth', '__missing__'),
-    ('py:attr', '__wrapped__'),
-    ('py:attr', 'decimal.Context.clamp'),
-    ('py:meth', 'index'),  # list.index, tuple.index, etc.
 ]
 
 # gh-106948: Copy standard C types declared in the "c:type" domain to the
@@ -293,9 +269,6 @@ html_theme_options = {
     'root_include_title': False   # We use the version switcher instead.
 }
 
-if os.getenv("READTHEDOCS"):
-    html_theme_options["hosted_on"] = '<a href="https://about.readthedocs.com/">Read the Docs</a>'
-
 # Override stylesheet fingerprinting for Windows CHM htmlhelp to fix GH-91207
 # https://github.com/python/cpython/issues/91207
 if any('htmlhelp' in arg for arg in sys.argv):
@@ -304,7 +277,7 @@ if any('htmlhelp' in arg for arg in sys.argv):
     print("It may be removed in the future\n")
 
 # Short title used e.g. for <title> HTML tags.
-html_short_title = f'{release} Documentation'
+html_short_title = '%s Documentation' % release
 
 # Deployment preview information
 # (See .readthedocs.yml and https://docs.readthedocs.io/en/stable/reference/environment-variables.html)
@@ -315,8 +288,9 @@ html_context = {
     "pr_id": os.getenv("READTHEDOCS_VERSION")
 }
 
-# This 'Last updated on:' timestamp is inserted at the bottom of every page.
-html_last_updated_fmt = time.strftime('%b %d, %Y (%H:%M UTC)', time.gmtime())
+# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
+# using the given strftime format.
+html_last_updated_fmt = '%b %d, %Y'
 
 # Path to find HTML templates.
 templates_path = ['tools/templates']
@@ -353,9 +327,12 @@ html_split_index = True
 
 latex_engine = 'xelatex'
 
+# Get LaTeX to handle Unicode correctly
 latex_elements = {
-    # For the LaTeX preamble.
-    'preamble': r'''
+}
+
+# Additional stuff for the LaTeX preamble.
+latex_elements['preamble'] = r'''
 \authoraddress{
   \sphinxstrong{Python Software Foundation}\\
   Email: \sphinxemail{docs@python.org}
@@ -363,12 +340,13 @@ latex_elements = {
 \let\Verbatim=\OriginalVerbatim
 \let\endVerbatim=\endOriginalVerbatim
 \setcounter{tocdepth}{2}
-''',
-    # The paper size ('letter' or 'a4').
-    'papersize': 'a4',
-    # The font size ('10pt', '11pt' or '12pt').
-    'pointsize': '10pt',
-}
+'''
+
+# The paper size ('letter' or 'a4').
+latex_elements['papersize'] = 'a4'
+
+# The font size ('10pt', '11pt' or '12pt').
+latex_elements['pointsize'] = '10pt'
 
 # Grouping the document tree into LaTeX files. List of tuples
 # (source start file, target name, title, author, document class [howto/manual]).
@@ -416,7 +394,6 @@ epub_publisher = 'Python Software Foundation'
 coverage_ignore_modules = [
     r'[T|t][k|K]',
     r'Tix',
-    r'distutils.*',
 ]
 
 coverage_ignore_functions = [
@@ -433,9 +410,9 @@ coverage_c_path = [
 
 # Regexes to find C items in the source files.
 coverage_c_regexes = {
-    'cfunction': r'^PyAPI_FUNC\(.*\)\s+([^_][\w_]+)',
-    'data': r'^PyAPI_DATA\(.*\)\s+([^_][\w_]+)',
-    'macro': r'^#define ([^_][\w_]+)\(.*\)[\s|\\]',
+    'cfunction': (r'^PyAPI_FUNC\(.*\)\s+([^_][\w_]+)'),
+    'data': (r'^PyAPI_DATA\(.*\)\s+([^_][\w_]+)'),
+    'macro': (r'^#define ([^_][\w_]+)\(.*\)[\s|\\]'),
 }
 
 # The coverage checker will ignore all C items whose names match these regexes
